(function() {
    'use strict';
        angular.module('pgApp')
            .controller('ProductGroupCtrl', ['$scope', '$q','appConstant', 'currentAccount',
                'currencyService', 'orderLineService', 'productService', 'productKitHeaderService',
                'productGroupService', 'productGroupProductsService', ProductGroupCtrl]);
        function ProductGroupCtrl($scope, $q, appConstant, currentAccount,
                                  currencyService, orderLineService, productService, productKitHeaderService,
                                  productGroupService, productGroupProductsService) {
            var dash = this;
            dash.productKitItemParentIds = [];
            dash.currencyIsoCode = "";
            dash.IsKitComponentsFixed = false;
            dash.products = [];
            dash.prodGroups = [];
            dash.prodGroupProds = [];
            dash.orderLines = [];
            $scope.quantityHeader = "Quantity";
            $scope.listHeader = "List Amount";
            $scope.netHeader = "Net Amount";
            getProductKitItems(currentAccount);

            function getProductKitItems(results){
                if (results.success) {
                    if(results.labels){
                        $scope.quantityHeader = results.labels[0].display;
                        $scope.listHeader = results.labels[1].display;
                        $scope.netHeader = results.labels[2].display;
                    }
                    for (var index in results.Product_vod__c){
                        dash.productKitItemParentIds.push(results.Product_vod__c[index].Parent_Product_vod__c);
                    }
                    getCurrencyIsoCode();
                }
            }

            function getCurrencyIsoCode(){
                var object = "Order";
                var field = "CurrencyIsoCode";
                getOrderFieldData(object, field).then(function(results1) {
                    if(results1 && results1.success){
                        dash.currencyIsoCode = results1[object].CurrencyIsoCode;
                    }
                    console.log(results1);
                    getOrderId();
                }, function(){
                    getOrderId();
                });
            }

            function getOrderId(){
                var object = "Order";
                var field = "ID";
                getOrderFieldData(object, field).then(function(results2) {
                    if (results2 && results2.success) {
                        var curOrderId = results2[object].ID;
                        getOrderLines(curOrderId);
                    } else {
                        setErrorMessage(results2.message);
                    }
                }, function(err){
                    console.log(err);
                });
            }

            function getOrderFieldData(object, Id){
                var deferred = $q.defer();
                currencyService.getCurrencyIsoCode(object, Id).then(function(results) {
                    deferred.resolve(results);
                }, function() {
                    deferred.reject();
                });
                return deferred.promise;
            }

            function getOrderLines(orderId){
                orderLineService.getOrderLines_Order(orderId).then(function(results) {
                    if (results.success) {
                        var prodIds = [];
                        for (var index in results.Order_Line_vod__c){
                            if(results.Order_Line_vod__c[index].Product_vod__c){
                                prodIds.push(results.Order_Line_vod__c[index].Product_vod__c);
                            }
                            dash.orderLines.push(results.Order_Line_vod__c[index]);
                        }

                        if (prodIds.length > 0){
                            var productIds = [];
                            for (var i = 0; i < prodIds.length; i++) {
                                productIds.push(prodIds[i]);
                            }
                            parse_getProducts_Results(productIds);
                        }
                    }
                    else{
                        setErrorMessage(results.message);
                    }
                }, function() {
                    setErrorMessage();
                });
            }

            function parse_getProducts_Results(productIds){
                productService.getProductsList(productIds).then(function(results) {
                    if (results.success) {
                        for (var index in results.Product_vod__c){
                            dash.products.push(results.Product_vod__c[index]);
                        }
                        setIsKitComponentsFixed();

                        if (!dash.IsKitComponentsFixed && dash.productKitItemParentIds.length > 0){
                            addKitHeaders();
                        }
                        else{
                            getProdGroups();
                        }
                    }
                    else {
                        setErrorMessage(results.message);
                    }
                }, function() {
                    setErrorMessage();
                });
            }

            function setIsKitComponentsFixed(){
                for (var prodIter = 0; prodIter < dash.products.length; prodIter++){
                    for (var prodIter2 = 0; prodIter2 < dash.products.length; prodIter2++){
                        if (dash.products[prodIter].Parent_Product_vod__c === dash.products[prodIter2].ID){
                            dash.IsKitComponentsFixed = true;
                            return;
                        }
                    }
                }
                dash.IsKitComponentsFixed = false;
            }

            function addKitHeaders(){
                var kitItemParentIds = [];

                for (var p = 0; p < dash.products.length; p++) {
                    if (dash.products[p].Product_Type_vod__c === 'Kit Item'){
                        if(dash.products[p].Parent_Product_vod__c){
                            kitItemParentIds.push(dash.products[p].Parent_Product_vod__c);
                        }
                    }
                }
                productKitHeaderService.getProductsKitHeaders(kitItemParentIds).then(function(results) {
                    if (results.success){
                        for (var index in results.Product_vod__c){
                            dash.products.push(results.Product_vod__c[index]);
                        }
                    } else {
                        setErrorMessage(results.message);
                    }
                    getProdGroups();
                }, function() {
                    setErrorMessage();
                });
            }

            function getProdGroups(){
                var productIds = [];
                if (dash.products.length > 0){
                    for (var i = 0; i < dash.products.length; i++) {
                        if (dash.products[i].ID){
                            productIds.push(dash.products[i].ID);
                        }
                    }
                    productGroupService.getProductGroup(productIds).then(function(results) {
                        if (results.success){
                            for (var index in results) {
                                dash.prodGroups.push(results[index]);
                            }
                            getProductGroupProducts();
                        }
                        else{
                            setErrorMessage(results.message);
                        }
                    }, function() {
                        setErrorMessage();
                    });
                }
            }

            function getProductGroupProducts(){
                var prodCatalogIds = [];

                for (var i = 0; i < dash.prodGroups.length; i++) {
                    var productCatalog = dash.prodGroups[i].Product_Catalog_vod__c;
                    if (productCatalog && prodCatalogIds.indexOf(productCatalog) === -1){
                        prodCatalogIds.push(productCatalog);
                    }
                }
                productGroupProductsService.getproductGroupProducts((prodCatalogIds)).then(function(results) {
                    if (results.success){
                        var prodGroupProdsWithSortCode = [];
                        var prodGroupProdsWithoutSortCode = [];

                        for (var index in results.Product_vod__c) {
                            if (results.Product_vod__c[index].Sort_Code_vod__c && results.Product_vod__c[index].Sort_Code_vod__c.length > 0){
                                prodGroupProdsWithSortCode.push(results.Product_vod__c[index]);
                            }
                            else{
                                prodGroupProdsWithoutSortCode.push(results.Product_vod__c[index]);
                            }
                        }
                        dash.prodGroupProds = prodGroupProdsWithSortCode.concat(prodGroupProdsWithoutSortCode);
                        updateTable();
                    }
                    else{
                        setErrorMessage(results.message);
                    }
                }, function() {
                    setErrorMessage();
                });

            }

            function updateTable(){
                var table = document.getElementById("resultTable");
                var rowIter = 1;

                var tableData = getTableData();
                for (var tableDataIndex in tableData){
                    var prodGrpQuantity = 0;
                    var prodGrpPrice = 0;
                    var prodGrpNetAmount = 0;
                    var tableDataRow = tableData[tableDataIndex];

                    if (tableDataRow.OrderLines.length > 0){
                        var prodGrp = tableDataRow.ProductGroup;
                        var prodGrpRow = table.insertRow(rowIter++);
                        var prodGrpCell0 = prodGrpRow.insertCell(0);
                        prodGrpCell0.innerHTML = prodGrp.Name;
                        prodGrpCell0.className = 'prodGroupCell';

                        console.log(tableDataRow);
                        for (var orderLineIndex in tableDataRow.OrderLines){
                            console.log(orderLineIndex);
                            var orderLine = tableDataRow.OrderLines[orderLineIndex];
                            if(orderLine) {
                                console.log(orderLine);
                                var prod = {};
                                if (orderLine && orderLine.Product_vod__c) {
                                    prod = getProd(orderLine.Product_vod__c);
                                }
                                if (!prod) {
                                    prod = {};
                                }
                                var row = table.insertRow(rowIter++);
                                insertCellIntoRow(row, 0, prod.Name);
                                insertCellIntoRow(row, 1, orderLine.Quantity_vod__c);
                                insertCellIntoRow(row, 2, dash.currencyIsoCode + " " + getFloat(orderLine.List_Amount_vod__c).toFixed(2));
                                insertCellIntoRow(row, 3, dash.currencyIsoCode + " " + getFloat(orderLine.Net_Amount_vod__c).toFixed(2));

                                prodGrpQuantity += parseInt(orderLine.Quantity_vod__c);
                                prodGrpPrice += getFloat(orderLine.List_Amount_vod__c);
                                prodGrpNetAmount += getFloat(orderLine.Net_Amount_vod__c);
                            }
                        }

                        var prodGrpCell1 = prodGrpRow.insertCell(1);
                        prodGrpCell1.innerHTML = prodGrpQuantity;
                        prodGrpCell1.className = 'prodGroupCell';

                        var prodGrpCell2 = prodGrpRow.insertCell(2);
                        prodGrpCell2.innerHTML = dash.currencyIsoCode + " " + prodGrpPrice.toFixed(2);
                        prodGrpCell2.className = 'prodGroupCell';

                        var prodGrpCell3 = prodGrpRow.insertCell(3);
                        prodGrpCell3.innerHTML = dash.currencyIsoCode + " " + prodGrpNetAmount.toFixed(2);
                        prodGrpCell3.className = 'prodGroupCell';
                    }
                }
            }

            function getTableData(){
                var tableData = [];

                for (var prodGroupProdIndex in dash.prodGroupProds){
                    var tableDataProdGrps = getProduct_GroupsForProductGroup(dash.prodGroupProds[prodGroupProdIndex]);
                    var tableDataOrderLines = getOrderLinesForProductGroups(tableDataProdGrps);
                    tableData.push({ ProductGroup : dash.prodGroupProds[prodGroupProdIndex], ProductGroups : tableDataProdGrps, OrderLines : tableDataOrderLines });
                }
                return tableData;
            }

            function getProduct_GroupsForProductGroup(prodGrp){
                var ret = [];
                for (var prodGroupIndex in dash.prodGroups){
                    if (dash.prodGroups[prodGroupIndex].Product_Catalog_vod__c === prodGrp.ID){
                        ret.push(dash.prodGroups[prodGroupIndex]);
                    }
                }
                return ret;
            }

            function checkDup(val, ret){
                for(var i=0; i< ret.length; i++){
                    if(val.ID === ret[i].ID){
                        return false;
                    }
                }
                return true;
            }

            function getOrderLinesForProductGroups(prodGrps){
                var ret = [];
                var val;
                for (var prodGroupIndex in prodGrps){
                    //if product is Kit Item Header
                    if (dash.productKitItemParentIds.indexOf(prodGrps[prodGroupIndex].Product_vod__c) >= 0){
                        //get Kit Items
                        for (var productsIndex in dash.products){
                            if (dash.products[productsIndex].Parent_Product_vod__c === prodGrps[prodGroupIndex].Product_vod__c){
                                if (dash.IsKitComponentsFixed){
                                    //Item must be Pricing Bound
                                    if (dash.products[productsIndex].Pricing_Bound_vod__c === true || dash.products[productsIndex].Pricing_Bound_vod__c === "True" || dash.products[productsIndex].Pricing_Bound_vod__c === 1){
                                        val = getOrderLine(dash.products[productsIndex].ID);
                                        if(val && checkDup(val, ret)) {
                                            ret.push(val);
                                        }

                                    }
                                }
                                else{
                                    val = getOrderLine(dash.products[productsIndex].ID);
                                    if(val && checkDup(val, ret)) {
                                        ret.push(val);
                                    }
                                }
                            }
                        }
                    }
                    else {
                        //if product is not Kit Header
                        var orderLineVal = getOrderLine(prodGrps[prodGroupIndex].Product_vod__c);
                        if (orderLineVal && checkDup(orderLineVal, ret)) {
                            ret.push(orderLineVal);
                        }
                    }
                }
                ret.sort(function(line1, line2){
                    var prod1 = getProd(line1.Product_vod__c);
                    var prod2 = getProd(line2.Product_vod__c);

                    return prod1.Name.localeCompare(prod2.Name);
                });
                return ret;
            }

            function getOrderLine(prodId){
                for (var orderLineIndex in dash.orderLines){
                    if (prodId === dash.orderLines[orderLineIndex].Product_vod__c){
                        return dash.orderLines[orderLineIndex];
                    }
                }
            }

            function getFloat(val){
                return parseFloat((val+"").replace(/,/g,'.'));
            }

            function insertCellIntoRow(row, cellIndex, value){
                var cell1 = row.insertCell(cellIndex);
                cell1.innerHTML = value;
            }

            function getProd(prodId){
                for (var p = 0; p < dash.products.length; p++) {
                    if (dash.products[p].ID === prodId){
                        return dash.products[p];
                    }
                }
            }

            function setErrorMessage(errorMessage){
                $scope.errorMessage =  (errorMessage ? JSON.stringify(errorMessage) : "");
            }

        }
})();
