(function() {
    'use strict';

    angular.module('pgApp')
        .factory('endpoints', ['$q', 'queries', factoryFn])
    ;

    function factoryFn($q, queries) {
        var endpoints = this;
        var ds = window.ds;

        endpoints.getproductGroupProducts = function(prodCatalogIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.productGroupProducts);

            if(angular.isDefined(prodCatalogIds) && prodCatalogIds.length > 0) {
                queryConfig.where += ds.getInStatement(prodCatalogIds);
            } else {
                queryConfig.where = null;
            }

            ds.queryRecord(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.getProductKitItems = function() {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.kitItems);
            ds.queryRecord(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.getCurrencyIsoCode = function(object, field) {
            var deferred = $q.defer();
            var accountPromise = ds.getDataForCurrentObject(object, field);
            accountPromise.then(function (resp) {
                deferred.resolve(resp);
            }, this.genericQueryErrorHandler);
            return accountPromise;
        };

        endpoints.getOrderLines_Order = function(orderId, bundlePack) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.orderLine);

            if(angular.isDefined(orderId)) {
                queryConfig.where = queryConfig.where.replace(/_orderId/, '\'' + orderId + '\'');
            } else {
                queryConfig.where = null;
            }
            if(bundlePack){
                queryConfig.where += " And (Bundle_Pack_Name_vod__c = null or Bundle_Pack_Name_vod__c = \'\')";
            }

            ds.queryRecord(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.getProductsList = function(productIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.productsList);

            if(angular.isDefined(productIds) && productIds.length > 0) {
                queryConfig.where += ds.getInStatement(productIds);
            } else {
                queryConfig.where = ""; //"{\'\'}";
            }
            ds.queryRecord(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.getProductsKitHeaders = function(kitItemParentIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.productKitHeaders);

            if(angular.isDefined(kitItemParentIds) && kitItemParentIds.length > 0) {
                queryConfig.where += ds.getInStatement(kitItemParentIds);
            } else {
                queryConfig.where = "";
            }
            ds.queryRecord(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.getProductGroup = function(productIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.productGroup);

            if(angular.isDefined(productIds) && productIds.length > 0) {
                queryConfig.where += ds.getInStatement(productIds);
            } else {
                queryConfig.where = "";
            }

            ds.queryRecord(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchCurrentUser = function() {
            var deferred = $q.defer();

            ds.getDataForCurrentObject('User', 'Id').then(function(resp1) {
                if(resp1.success) {
                    var currentUser = resp1.User;
                    var queryConfig = angular.copy(queries.user);
                    queryConfig.where = queryConfig.where.replace(/_userId/, '\'' + currentUser.Id + '\'');

                    ds.queryRecord(queryConfig).then(function (resp2) {
                        deferred.resolve(resp2);
                    }, genericQueryErrorHandler(deferred));

                } else {
                    (genericQueryErrorHandler(deferred))(resp1.message);
                }
            });

            return deferred.promise;
        };

        endpoints.fetchLabels= function() {
            var deferred = $q.defer();

            var queryConfig = angular.copy(queries.labels);
            ds.getFieldLabels(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));


            return deferred.promise;
        };

        endpoints.checkBundlePack = function() {
            var deferred = $q.defer();

            var queryConfig = angular.copy(queries.bundlePack);
            ds.getFieldLabels(queryConfig).then(function (resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));


            return deferred.promise;
        };

        /**
         * Private functions
         */
        function genericQueryErrorHandler(deferred) {
            return function(error) {
                deferred.reject(error);
            };
        }

        return endpoints;
    }
})();